/***************************************************************
 * Name:      wxiscanframe.h
 * Purpose:   Defines Application Frame
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2008-05-04
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

#ifndef WXISCANMAIN_H
#define WXISCANMAIN_H

//(*Headers(wxIScanFrame)
#include <wx/toolbar.h>
#include <wx/menu.h>
#include <wx/scrolwin.h>
#include <wx/splitter.h>
#include <wx/statusbr.h>
#include <wx/frame.h>
#include <wx/textctrl.h>
//*)

// Additional wxWidgets header files
#include <wx/docview.h>
#include <wx/process.h>

// Private header files.
#include "wxextfiledroptarget.h"
#include "wxiviewcanvas.h"
#include "wxiviewpaint.h"
#if __WXLUASCRIPT__
#   include "wxluascript.h"
#endif // __WXLUASCRIPT__


// IDs
#define wxISCAN_PROCESS     1


//////////////////////////////////////////////////////////
// Predefined classes
//
#if __WXPDFDOC__
class wxPdfDocument;
class wxXmlDocument;
class wxXmlNode;
#endif // __WXPDFDOC__
class wxExtProcess;
#if __WXLUASCRIPT__
struct lua_State;
#endif // __WXLUASCRIPT__


#if __WXLUASCRIPT__
/////////////////////////////////////////////////////////////////////////////
//  Class wxIScanLuaScript
//
/// \brief An extended class to handle Lua scripting.
///
/// wxIScanLuaScript is a friend class to the frame window class
/// wxIScanFrame an is intended to control the frame window
/// remotely.
class wxIScanLuaScript : public wxLuaScript
{
  public:
    /// \brief Standard constructor.
    ///
    /// \param strScript     ...
    /// \param wxEvtHandler  A pointer to the parent event handler
    ///                      (mostly the parent window).
    /// \param pbParentState ...
    wxIScanLuaScript( const wxString& strScript= wxEmptyString,
                      wxEvtHandler *poParent= NULL, bool *pbParentState= NULL );

  private:
    // Addtional Lua functions.
    /// \brief Do OCR on the loaded/scanned image and return the text (to Lua).
    ///
    /// \param L      a pointer to the Lua context.
    static int OCR( lua_State* L );

    /// \brief Open an image file in Lua using a given name.
    ///
    /// \param L      a pointer to the Lua context.
    static int OpenImage( lua_State* L );

    /// \brief Save an image file in Lua using a given name.
    ///
    /// \param L      a pointer to the Lua context.
    static int SaveImage( lua_State* L );

    /// \brief Copy a given text to the clipboard in Lua.
    ///
    /// \param L      a pointer to the Lua context.
    static int CopyText2Clipboard( lua_State* L );

    /// \brief Replace the program's image by its monochrome version.
    ///
    /// \param L      a pointer to the Lua context.
    static int Image2Mono( lua_State* L );

    /// \brief Replace the program's image by it version that is colour enhenced.
    ///
    /// \param L      a pointer to theLua context.
    static int EnhenceColours( lua_State* L );

    /// \brief Replace white colour by yellow.
    ///
    /// \param L      a pointer to the Lua context.
    static int EnhenceOpticalContrast( lua_State* L );

    /// \brief (Fine) Rotate an image by a given angle.
    ///
    /// \param L      a pointer to the Lua context.
    static int RotateImage( lua_State* L );

    /// \brief Fork an external process.
    ///
    /// \param L      a pointer to the Lua context.
    static int Execute( lua_State* L );

    /// \brief Set the filename of the image to "unknown.XXX".
    ///
    /// \param L      a pointer to the Lua context.
    static int ResetFileName( lua_State* L );
};
#endif // __WXLUASCRIPT__


//////////////////////////////////////////////////////////
// Class wxIScanFrame
//
/// \brief The frame window class.
///
/// ...
///
/// Note:
///         Additionaly to wxFrame wxIScanFrame inherits the
///         class wxIViewPaintBase which is an interface to
///         basic printing and print previewing of an image.
class wxIScanFrame: public wxFrame, public wxIViewPaintBase
{
    public:
        enum
        {
            wxCANVASFULLSIZE,                                       ///< Show image full size (without any zooming).
            wxCANVASFITWIDTH,                                       ///< Zoom image to fit the window width.
            wxCANVASFITWINDOW,                                      ///< Zoom image to fit the window size.
            wxCANVASCUSTOMSIZE                                      ///< Zoom image to an explicit given zoom factor (m_nScaleFactor).
        };

    public:
        /// \brief Standard constructor.
        ///
        /// \param poParent                  a pointer to the parent window
        /// \param nId                       the window id
        wxIScanFrame( wxWindow* poParent, wxWindowID nId= wxID_ANY );

        /// \brief Virtual destructor.
        virtual ~wxIScanFrame();

        virtual bool CanClose( bool bCanVeto= true );               ///< Handle window closing requests.
        virtual bool OpenImage( const wxString &strFileName );      ///< Open an image using the given name.
        virtual bool OnDocumentPageChanged();                       ///< (Re)loads the (zero based) page number given in m_nPage.
        virtual void SetBitmap();                                   ///< ...
        virtual wxImage GetImageSelection();                        ///< Get the selected sub image.
        virtual bool ScanImage();                                   ///< Get an image from scanner.
        virtual void SetProfile( int nProfileIndex );               ///< Select a profile (by number).
        virtual void ApplyProfile();                                ///< Apply the selected profile to the image.

        /// \brief Enhance the optical contrast by replacing white by yellow.
        virtual void EnhenceOpticalContrast()
        {
            m_oImage.Replace( 255, 255, 255, 255, 255, 0 );
            SetBitmap();
        }

        /// \brief Call an external OCR software an return the
        ///        filename of the resulting text file.
        ///
        /// \param strTempFileName           the file name of the image that
        ///                                  should be read by OCR().
        /// \param bUseHocr                  ..., defaults to false.
        virtual wxString OCR( const wxString &strTempFileName, bool bUseHocr= false );

        virtual bool ReadOut( bool bAsync= true );                  ///< Do OCR and read out the returned text.

        /// \brief Fork an external process.
        ///
        /// \param strCommand                the command line to be executed.
        /// \param bAsync                    an (optional) flag which decides if the new process
        ///                                  is to be executed synchronous (default) or asynchronous.
        /// \param strTempTextFileName       an (optional) file name of a file that should be deleted
        ///                                  at the end of an asynchronous running process.
        virtual bool Execute( const wxString &strCommand, bool bAsync= false,
                              const wxString &strTempTextFileName= wxEmptyString );

#if __WXPDFDOC__
        virtual bool CreateNewPdfDocument();                        ///< Create (and initialize) a new PDF document.
        virtual bool AddPdfPage();                                  ///< Add a new image/OCR text page to the existing PDF document.


        /// \brief ...
        virtual bool IsPdfDirty()
        {
            return m_bPdfDirty;
        }

#endif // __WXPDFDOC__
        virtual wxString GetTextFromFile( const wxString &strFilename ); ///< Load the content of a text file into memory.
        virtual bool CopyText2Clipboard( const wxString &strText ); ///< Copy the given text to the sytem clipboard.
        virtual bool CopyImage2Clipboard( const wxImage &oImage );  ///< Copy the given image to the sytem clipboard.

        /// \brief State if the image is in portrait direction.
        virtual bool IsPortrait()
        {
            return ( m_oImage.GetHeight() > m_oImage.GetWidth() );
        }

        virtual void ResetFileName();                               ///< Reset the filename to "unknown.jpg" etc.

        /// \brief Toggle full screen mode.
        virtual void ToggleFullScreen()
        {
            ShowFullScreen( !IsFullScreen(),
                             wxFULLSCREEN_NOSTATUSBAR | wxFULLSCREEN_NOBORDER | wxFULLSCREEN_NOCAPTION );
        }

        virtual void RestoreSettings();                             ///< Restore program settings from config file.
        virtual void SaveSettings();                                ///< Save program settings in config file.
        virtual void EnableControls();                              ///< Enable or disable controls according to the actual progam state.

        /// \brief ...
        virtual void LockUI( bool bLock= true )
        {
            m_bUILocked= bLock; EnableControls();
        }

        /// \brief ...
        virtual void UnlockUI()
        {
            LockUI( false );
        }

#if __WXLUASCRIPT__
        /// \brief Start a Lua script from a give file.
        ///
        /// \param strScriptFileName         file name of the text file containing the Lua script
        /// \param strScriptStartupFunction  startup function of the Lua script
        virtual void StartScript( const wxString &strScriptFileName,
                                  const wxString &strScriptStartupFunction );
#endif // __WXLUASCRIPT__

#if __WXPDFDOC__
    private:
        // Private helper functions.
        void AddPdfPageTraverseXmlNodes( wxXmlNode *poNode );       ///< DOM tree traverse helper function for AddPdfPage().
#endif // __WXPDFDOC__

    public:
        // Printer handling (introduced by base class wxIViewPaintBase)
        virtual void AdjustOrientation(){ m_oPrintData.SetOrientation( IsPortrait() ? wxPORTRAIT : wxLANDSCAPE ); } ///< ...
        virtual bool GetPageSizeMM( int *pnWidth, int *pnHeight );  ///< Calculates the image extents in mm while printing or print previewing.
        virtual int  GetMaxPage(){ return m_nImageCount; }          ///< Get the (zero based) page number of the last page.
        virtual bool GotoPage( int nPage );                         ///< Go to a given printer page.
        virtual bool Paint( wxDC& oDc );                            ///< Paints the image while printing or print previewing.

    private:
        // Member variables
        bool m_bUILocked;                                           ///< States if the GUI should be locked by wxIScanFrame::EnableControls().
        wxImage m_oImage;                                           ///< Image object.
        wxImage m_oPrintImage;                                      ///< Image object for printing.
        wxString m_strImageFileName;                                ///< Image file name.
        int m_nPage;                                                ///< Page number.
        int m_nImageCount;                                          ///< The number of images in an image file.
        int m_nResolution;                                          ///< The (fictive) resolution of an image in dpi.

        // Options
        int m_nFit;                                                 ///< Fit bitmap size according to its value.
        double m_nScaleFactor;                                      ///< Bitmap zoom factor.
        bool m_bAutoApplyProfile;                                   ///< Apply a profile automatically after scanning.
        bool m_bAutoApplyProfileOnMultiPage;                        ///< Apply a profile automatically after loading a page from a multi page file.
        int m_nProfileIndex;                                        ///< Index number of the current image profile.
        wxString m_strBitmapMimeType;                               ///< Image mimetype for any export but PDF export.
        wxString m_strPdfBitmapMimeType;                            ///< Image mimetype for PDF export.
#if __WXLUASCRIPT__
        bool m_bLua;                                                ///< Enable Lua scripting.
#endif // __WXLUASCRIPT__
        bool m_bOCR;                                                ///< Enable OCR.
        wxString m_strOcrOutputExtension;                           ///< The output extension of the via OCR extracted text.
        bool m_bCmdConvertEnable;                                   ///< ...
        wxString m_strCmdConvert;                                   ///< ...
        bool m_bCmdOcrEnable;                                       ///< ...
        wxString m_strCmdOcr;                                       ///< ...
        bool m_bCmdCleanUpEnable;                                   ///< ...
        wxString m_strCmdCleanUp;                                   ///< ...
        bool m_bHocrEnable;                                         ///< ...
        bool m_bSmartHocrEnable;                                    ///< ...
        wxString m_strCmdOcrUseHocr;                                ///< ...
        bool m_bCmdHtml2XhtmlEnable;                                ///< ...
        wxString m_strCmdHtml2Xhtml;                                ///< ...
        bool m_bCmdHocrCleanUpEnable;                               ///< ...
        wxString m_strCmdHocrCleanUp;                               ///< ...
        bool m_bCmdTtsEnable;                                       ///< ...
        wxString m_strCmdTts;                                       ///< ...
        bool m_bCmdEditEnable;                                      ///< ...
        wxString m_strCmdEdit;                                      ///< ...

        // File history
        wxFileHistory m_oFileHistory;                               ///< History of last opened files

        // Printing support.
        wxPrintData m_oPrintData;                                   ///< Holds related to printers and printer device contexts.

#if __WXPDFDOC__
        // PDF support
        wxPdfDocument *m_poPdfDoc;                                  ///< PDF document.
        int m_nPdfPages;                                            ///< Number of PDF pages in memory.
        bool m_bPdfDirty;                                           ///< States if the PDF file is dirty (and should be saved).
#endif // __WXPDFDOC__

#if __WXLUASCRIPT__
        bool bLuaReturnFromEventHandler;                            ///< Lua return value of the event handlers.
#endif // __WXLUASCRIPT__

        // ...
        wxExtProcess *m_poRunningProcess;                           ///< A concurrent running process (equals to NULL if there isn't any).

    private:
        //(*Handlers(wxIScanFrame)
        void OnFileQuit(wxCommandEvent& event);
        void OnFileScanImage(wxCommandEvent& event);
        void OnFileOpen(wxCommandEvent& event);
        void OnFileSaveAs(wxCommandEvent& event);
        void OnFilePrint( wxCommandEvent &oEvent );
        void OnFilePrintPreview( wxCommandEvent &oEvent );
        void OnFilePrintSetup( wxCommandEvent &oEvent );
        void OnEditCopy(wxCommandEvent& event);
        void OnPdfBeginPdf(wxCommandEvent& event);
        void OnDocumentCopyText(wxCommandEvent& event);
        void OnDocumentReadText(wxCommandEvent& event);
        void OnDocumentFirstPage(wxCommandEvent& event);
        void OnDocumentPreviousPage(wxCommandEvent& event);
        void OnDocumentNextPage(wxCommandEvent& event);
        void OnDocumentLastPage(wxCommandEvent& event);
        void OnDocumentRotate90Left(wxCommandEvent& event);
        void OnDocumentRotate90Right(wxCommandEvent& event);
        void OnDocumentApplyProfile(wxCommandEvent& event);
        void OnPdfAddPage(wxCommandEvent& event);
        void OnPdfSavePdf(wxCommandEvent& event);
        void OnHelpAbout(wxCommandEvent& event);
        void OnCmdStop(wxCommandEvent& event);
        void OnClose(wxCloseEvent& event);
        void OnEditPaste(wxCommandEvent& event);
        void OnEditSettings(wxCommandEvent& event);
        void OnEditSettingsFile(wxCommandEvent& event);
        void OnDocumentCrop(wxCommandEvent& event);
        void OnEditClearHistory(wxCommandEvent& event);
        void OnEditClearLog(wxCommandEvent& event);
        void OnDocumentZoom(wxCommandEvent& event);
        void OnViewFit(wxCommandEvent& event);
        void OnViewFullScreen(wxCommandEvent& event);
        //*)
        void OnDocumentProfilesChooseProfile( wxCommandEvent& oEvent ); ///< Handle menu command "Document|Profile|[PROFILE #X]".
#if __WXLUASCRIPT__
        void OnScriptStartScript( wxCommandEvent& oEvent );         ///< Handle menu command "Scrips|[SCRIPT NAME]".
#endif // __WXLUASCRIPT__
        void OnMRUFiles( wxCommandEvent& oEvent );                  ///< Handle menu command "File|[MOST RECENTLY USED FILE #X]".
        void OnFileDrop( wxExtFileDropEvent& oEvent );              ///< Handle file dropping.
        void OnCanvasSize( wxIViewCanvasSizeEvent& oEvent );        ///< Handle bitmap resizing introduced by canvas resizing.
        void OnCanvasSelected( wxIViewCanvasSelectedEvent& oEvent ); ///< Handle canvas selection events.
        void OnProcessTerminated( wxProcessEvent& oEvent );         ///< Handle process termination.

        //(*Identifiers(wxIScanFrame)
        static const long ID_CANVAS;
        static const long ID_LOGTEXTCTRL;
        static const long ID_HSPLITTERWINDOW;
        static const long ID_FILESCANIMAGE;
        static const long ID_FILEOPEN;
        static const long ID_FILESAVEAS;
        static const long ID_FILEPRINT;
        static const long ID_FILEPRINTPREVIEW;
        static const long ID_FILEPRINTSETUP;
        static const long ID_FILEQUIT;
        static const long ID_EDITCOPY;
        static const long ID_EDITPASTE;
        static const long ID_EDITSETTINGS;
        static const long ID_EDITSETTINGSFILE;
        static const long ID_EDITCLEARLOG;
        static const long ID_EDITCLEARHISTORY;
        static const long ID_VIEWFITWIDTH;
        static const long ID_VIEWFITWINDOW;
        static const long ID_VIEWFULLSIZE;
        static const long ID_VIEWCUSTOMSIZE;
        static const long ID_VIEWFULLSCREEN;
        static const long ID_DOCUMENTCOPYTEXT;
        static const long ID_DOCUMENTREADTEXT;
        static const long ID_CMDSTOP;
        static const long ID_DOCUMENTFIRSTPAGE;
        static const long ID_DOCUMENTPREVIOUSPAGE;
        static const long ID_DOCUMENTNEXTPAGE;
        static const long ID_DOCUMENTLASTPAGE;
        static const long ID_DOCUMENTROTATE90LEFT;
        static const long ID_DOCUMENTROTATE90RIGHT;
        static const long ID_DOCUMENTCROP;
        static const long ID_DOCUMENTZOOM;
        static const long ID_DOCUMENTAPPLYPROFILE;
        static const long ID_MENUDOCUMENTPROFILES;
        static const long ID_PDFBEGINPDF;
        static const long ID_PDFADDPAGE;
        static const long ID_PDFSAVEASPDF;
        static const long ID_HELPABOUT;
        static const long ID_STATUSBAR;
        static const long ID_FILESCANIMAGE_TB;
        static const long ID_FILEOPEN_TB;
        static const long ID_FILESAVEAS_TB;
        static const long ID_FILEPRINT_TB;
        static const long ID_PDFBEGINPDF_TB;
        static const long ID_PDFADDPAGE_TB;
        static const long ID_PDFSAVEASPDF_TB;
        static const long ID_DOCUMENTFIRSTPAGE_TB;
        static const long ID_DOCUMENTPREVIOUSPAGE_TB;
        static const long ID_DOCUMENTNEXTPAGE_TB;
        static const long ID_DOCUMENTLASTPAGE_TB;
        static const long ID_DOCUMENTREADTEXT_TB;
        static const long ID_CMDSTOP_TB;
        static const long ID_TOOLBAR;
        //*)
        static const long ID_DOCUMENTREADTEXTSYNC;
        static const long ID_DOCUMENTPROFILE0;


        //(*Declarations(wxIScanFrame)
        wxIViewCanvas* m_poCanvas;
        wxToolBarToolBase* ToolBarItem5;
        wxMenu* Menu1;
        wxToolBarToolBase* ToolBarItem11;
        wxToolBarToolBase* ToolBarItem6;
        wxToolBarToolBase* ToolBarItem12;
        wxStatusBar* m_poStatusBar;
        wxToolBarToolBase* ToolBarItem7;
        wxToolBarToolBase* ToolBarItem2;
        wxToolBarToolBase* ToolBarItem9;
        wxToolBarToolBase* ToolBarItem10;
        wxToolBarToolBase* ToolBarItem4;
        wxToolBarToolBase* ToolBarItem1;
        wxToolBarToolBase* ToolBarItem3;
        wxSplitterWindow* m_poHSplitterWindow;
        wxTextCtrl* m_poLogTextCtrl;
        wxToolBarToolBase* ToolBarItem8;
        //*)

        // Friend classes.
#if __WXLUASCRIPT__
        friend class wxIScanLuaScript;
#endif // __WXLUASCRIPT__

        // Event table.
        DECLARE_EVENT_TABLE()
};


#if __WXPDFDOC__
//////////////////////////////////////////////////////////
// Class wxIScanFrameHocr2Pdf
//
/// \brief DOM tree traverser class for
///        wxIScanFrame::AddPdfPage().
///
/// This is a helper class that traverses a hOCR DOM tree
/// and "prints" the text of a hOCR XML file hidden behind
/// the corresponding point on the image.
///
/// NOTE:
///
/// 1) This is a "private" class and should not be called
///    outside of AddPdfPage().
///
/// 2) There is no validity check done on poXmlDoc, poPdfDoc
///    and nResolution. That is it is asumed all parameter
///    of the constructor ared valid.
class wxIScanHocr2Pdf
{
  public:
    /// \brief Standard constructor.
    ///
    /// \param poXmlDoc     the (valid!) pointer to the XML DOM tree
    /// \param poPdfDoc     the (valid!) pointer to the PDF document
    /// \param nResolution  the (virtual) resolution of the image
    wxIScanHocr2Pdf( wxXmlDocument *poXmlDoc, wxPdfDocument *poPdfDoc, int nResolution );

    /// \brief Virtual destructor;
    ~wxIScanHocr2Pdf(){}

    /// \brief Traverse the DOM tree by calling TraverseXmlNodes()
    ///        and flushing outstanding operations in the end.
    virtual void Run();

  protected:

    /// \brief Traverse recursively through the DOM tree
    ///        beginning with the given node.
    ///
    /// \param poNode      pointer to start node in the DOM tree.
    virtual void TraverseXmlNodes( wxXmlNode *poNode );

    /// \brief Print the given text at the given coordinates.
    ///
    /// \param x            abscissa of the origin
    /// \param y            ordinate of the origin
    /// \param strText      text to print
    ///
    /// NOTE:
    ///        If you want to change the behaviour of the text placement
    ///        you should override this function.
    virtual void Print2Pdf( double x, double y, const wxString& strText );

    /// \brief Flush outstanding print-to-PDF-commands.
    ///
    /// NOTE:
    ///        This function does nothing, but can be overriden.
    virtual void Flush2Pdf(){}

  protected:
    wxXmlDocument *m_poXmlDoc;          ///< The pointer to the XML document.
    wxPdfDocument *m_poPdfDoc;          ///< The pointer to the PDF document.
    int m_nResolution;                  ///< The (fictive) resolution of an image in dpi.
};

//////////////////////////////////////////////////////////
// Class wxIscanSmoothHocr2Pdf
//
/// \brief Traverse recursively through the DOM tree beginning with
///        the given node.
///
/// This class extends wxIScanHocr2Pdf by a "smart" glueing
/// the letters of one word together. This makes it easer to
/// search a PDF document for words.
class wxIscanSmartHocr2Pdf: public wxIScanHocr2Pdf
{
  public:
    /// \brief DOM tree traverser class for wxIScanFrame::AddPdfPage()
    ///        based on wxIScanFrameHocr2Pdf class to achieve a
    ///        "smarter" word based PDF output hidden behind the image.
    ///
    /// \param poXmlDoc     the (valid!) pointer to the XML DOM tree
    /// \param poPdfDoc     the (valid!) pointer to the PDF document
    /// \param nResolution  the (virtual) resolution of the image
    wxIscanSmartHocr2Pdf( wxXmlDocument *poXmlDoc, wxPdfDocument *poPdfDoc, int nResolution );

  protected:
    /// \brief Print the given text at the given coordinates.
    ///
    /// \param x            abscissa of the origin
    /// \param y            ordinate of the origin
    /// \param strText      text to print
    ///
    /// NOTE:
    ///        This function only collects text until Flush2Pdf()
    ///        is called.
    virtual void Print2Pdf( double x, double y, const wxString& strText );

    /// \brief Flush outstanding print-to-PDF-commands.
    ///
    /// NOTE:
    ///        This function does the real printing initialized by
    ///        (maybe multiple) calls to Print2Pdf().
    virtual void Flush2Pdf();

  protected:
    double m_x, m_y;                    ///< Save printing coordinates.
    wxString m_strText;                 ///< Save outstanding text.
};
#endif // __WXPDFDOC__

#endif // WXISCANMAIN_H
